"use strict";
/**
 * @module Client
 */
Object.defineProperty(exports, "__esModule", { value: true });
var internal_session_impl_1 = require("client/internal-session-impl");
var service_registry_1 = require("client/service-registry");
var monitored_ping_service_1 = require("client/services/monitored-ping-service");
var notify_subscription_service_1 = require("client/services/notify-subscription-service");
var notify_unsubscription_service_1 = require("client/services/notify-unsubscription-service");
var ping_service_1 = require("client/services/ping-service");
var ConversationSet = require("conversation/conversation-set");
var Services = require("services/services");
var ConnectionFactory = require("v4-stack/connection-factory");
/**
 * Create a new internal session
 *
 * @param options  the options passed to the session
 * @return         a new internal session
 */
function create(options) {
    var serviceRegistry = new service_registry_1.ServiceRegistry();
    // assign default service implementations
    serviceRegistry.add(Services.USER_PING, ping_service_1.pingService);
    serviceRegistry.add(Services.SYSTEM_PING, monitored_ping_service_1.monitoredPingService);
    serviceRegistry.add(Services.NOTIFY_SUBSCRIPTION, notify_subscription_service_1.notifySubscriptionService);
    serviceRegistry.add(Services.UNSUBSCRIPTION_NOTIFICATION, notify_unsubscription_service_1.notifyUnsubscriptionService);
    return new internal_session_impl_1.InternalSessionImpl(ConversationSet.create, serviceRegistry, ConnectionFactory.create, options);
}
exports.create = create;
