"use strict";
/**
 * @module Services.Send
 */
Object.defineProperty(exports, "__esModule", { value: true });
var Codec = require("io/codec");
var BEES = require("serialisers/byte-encoded-enum-serialiser");
var messages_1 = require("../../../features/messages");
var send_options_1 = require("./send-options");
/**
 * Read a {@link SendOptions} from the stream
 *
 * @param bis  the input stream
 * @return     the {@link SendOptions} that was read
 */
function read(bis) {
    // tslint:disable-next-line:deprecation
    var priority = BEES.read(bis, messages_1.Priority);
    var headers = Codec.readCollection(bis, Codec.readString);
    return new send_options_1.SendOptions(priority, headers);
}
exports.read = read;
/**
 * Write a {@link SendOptions} to the stream
 *
 * @param bos    the output stream
 * @param value  the {@link SendOptions} to be written
 */
function write(bos, value) {
    BEES.write(bos, value.priority);
    Codec.writeCollection(bos, value.headers, Codec.writeString);
}
exports.write = write;
