"use strict";
/**
 * @module Services.Send
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var ContentSerialiser = require("content/serialiser");
var conversation_id_serialiser_1 = require("conversation/conversation-id-serialiser");
var Codec = require("io/codec");
var serialiser_1 = require("serialisers/serialiser");
var session_id_serialiser_1 = require("session/session-id-serialiser");
var OptionsSerialiser = require("./send-options-serialiser");
var session_forward_send_request_1 = require("./session-forward-send-request");
/**
 * Serialiser for {@link SessionForwardSendRequest}
 */
var SessionForwardSendRequestSerialiserClass = /** @class */ (function (_super) {
    __extends(SessionForwardSendRequestSerialiserClass, _super);
    function SessionForwardSendRequestSerialiserClass() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * Read a {@link SessionForwardSendRequest} from the stream
     *
     * @param bis  the input stream
     * @return     the {@link SessionForwardSendRequest} that was read
     */
    SessionForwardSendRequestSerialiserClass.prototype.read = function (bis) {
        var path = Codec.readString(bis);
        var content = ContentSerialiser.read(bis);
        var sender = session_id_serialiser_1.SessionIdSerialiser.read(bis);
        var options = OptionsSerialiser.read(bis);
        var sessionProperties = Codec.readDictionary(bis, Codec.readString);
        var cid = conversation_id_serialiser_1.CIDSerialiser.read(bis);
        return new session_forward_send_request_1.SessionForwardSendRequest(cid, path, content, sender, sessionProperties, options);
    };
    /**
     * Write a {@link SessionForwardSendRequest} to the stream
     *
     * @param bos    the output stream
     * @param value  the {@link SessionForwardSendRequest} to be written
     */
    SessionForwardSendRequestSerialiserClass.prototype.write = function (bos, value) {
        Codec.writeString(bos, value.path);
        ContentSerialiser.write(bos, value.content);
        session_id_serialiser_1.SessionIdSerialiser.write(bos, value.sender);
        OptionsSerialiser.write(bos, value.options);
        Codec.writeDictionary(bos, value.sessionProperties, Codec.writeString);
        conversation_id_serialiser_1.CIDSerialiser.write(bos, value.cid);
    };
    return SessionForwardSendRequestSerialiserClass;
}(serialiser_1.AbstractSerialiser));
exports.SessionForwardSendRequestSerialiserClass = SessionForwardSendRequestSerialiserClass;
/**
 * The {@link SessionForwardSendRequestSerialiser} singleton
 */ // tslint:disable-next-line:variable-name
exports.SessionForwardSendRequestSerialiser = new SessionForwardSendRequestSerialiserClass();
