"use strict";
/**
 * @module Topics
 */
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Attributes for topic details
 *
 * This extracts a subset of the {@link TopicSpecification.properties} into
 * separate variables and
 */
var TopicDetailsAttributes = /** @class */ (function () {
    function TopicDetailsAttributes() {
        /**
         * The Slave Master Topic
         *
         * Type-specific: only serialised for slave topics
         *
         * @see {@link TopicSpecification.SLAVE_MASTER_TOPIC}
         */
        this.slaveMasterTopic = '';
        /**
         * The value for an empty field
         *
         * Type-specific: only serialised for record topics
         */
        this.emptyFieldValue = '';
        /**
         * A handler
         *
         * Type-specific: only serialised for routing topics
         */
        this.handler = '';
        /**
         * Tidy the topic on unsubscribe.
         *
         * @see {@link TopicSpecification.TIDY_ON_UNSUBSCRIBE}
         */
        this.tidyOnUnsubscribe = false;
        /**
         * Additional topic properties
         *
         * @see {@link TopicSpecification.properties}
         */
        this.properties = {};
        /**
         * A topic reference
         */
        this.reference = '';
    }
    return TopicDetailsAttributes;
}());
exports.TopicDetailsAttributes = TopicDetailsAttributes;
/**
 * Create topic details
 *
 * @param  type        the topic type
 * @param  schema      the schema as a JSON object
 * @param  properties  the topic properties
 * @return             topic details constructed from the parameters
 */
function createDetails(type, schema, properties) {
    var attributes = new TopicDetailsAttributes();
    // handle properties from TopicSpecifications
    if (properties) {
        var props = {};
        try {
            for (var _a = __values(Object.getOwnPropertyNames(properties)), _b = _a.next(); !_b.done; _b = _a.next()) {
                var p = _b.value;
                switch (p) {
                    case 'TIDY_ON_UNSUBSCRIBE':
                        attributes.tidyOnUnsubscribe = properties.TIDY_ON_UNSUBSCRIBE === 'true';
                        break;
                    case 'SLAVE_MASTER_TOPIC':
                        attributes.slaveMasterTopic = properties.SLAVE_MASTER_TOPIC;
                        break;
                    default:
                        props[p] = properties[p];
                        break;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_1) throw e_1.error; }
        }
        attributes.properties = props;
    }
    return {
        type: type,
        schema: schema,
        attributes: attributes
    };
    var e_1, _c;
}
/**
 * Create topic details from a topic specification
 *
 * @param  specification  the topic specification
 * @return                the corresponding topic details
 */
function detailsFromSpecification(specification) {
    return createDetails(specification.type, {}, specification.properties);
}
exports.detailsFromSpecification = detailsFromSpecification;
