"use strict";
/**
 * @module Topics
 */
Object.defineProperty(exports, "__esModule", { value: true });
var Codec = require("io/codec");
var TopicDetailsSerialiser = require("topics/details/topic-details-serialiser");
var TopicIdSerialiser = require("topics/details/topic-id-serialiser");
/**
 * Read topic information from the stream
 *
 * @param bis the input stream
 * @return    the topic information that was read
 */
function read(bis) {
    return {
        id: TopicIdSerialiser.read(bis),
        path: Codec.readString(bis),
        details: TopicDetailsSerialiser.read(bis)
    };
}
exports.read = read;
/**
 * Write topic information to the stream
 *
 * @param bos   the output stream
 * @param value the topic information to be written
 */
function write(bos, value) {
    TopicIdSerialiser.write(bos, value.id);
    Codec.writeString(bos, value.path);
    TopicDetailsSerialiser.write(bos, value.details);
}
exports.write = write;
