"use strict";
/**
 * @module Topics
 */
Object.defineProperty(exports, "__esModule", { value: true });
var KEY_MAP;
(function (KEY_MAP) {
    KEY_MAP[KEY_MAP["PUBLISH_VALUES_ONLY"] = 30] = "PUBLISH_VALUES_ONLY";
    KEY_MAP[KEY_MAP["VALIDATE_VALUES"] = 31] = "VALIDATE_VALUES";
    KEY_MAP[KEY_MAP["TIME_SERIES_EVENT_VALUE_TYPE"] = 2] = "TIME_SERIES_EVENT_VALUE_TYPE";
    KEY_MAP[KEY_MAP["TIME_SERIES_RETAINED_RANGE"] = 3] = "TIME_SERIES_RETAINED_RANGE";
    KEY_MAP[KEY_MAP["TIME_SERIES_SUBSCRIPTION_RANGE"] = 4] = "TIME_SERIES_SUBSCRIPTION_RANGE";
    KEY_MAP[KEY_MAP["SCHEMA"] = 5] = "SCHEMA";
    KEY_MAP[KEY_MAP["DONT_RETAIN_VALUE"] = 6] = "DONT_RETAIN_VALUE";
    KEY_MAP[KEY_MAP["PERSISTENT"] = 7] = "PERSISTENT";
    KEY_MAP[KEY_MAP["REMOVAL"] = 8] = "REMOVAL";
    KEY_MAP[KEY_MAP["_CREATOR"] = 9] = "_CREATOR";
    KEY_MAP[KEY_MAP["CONFLATION"] = 10] = "CONFLATION";
    KEY_MAP[KEY_MAP["OWNER"] = 11] = "OWNER";
})(KEY_MAP || (KEY_MAP = {}));
var Codec = require("io/codec");
/**
 * Read Topic Properties from the stream
 *
 * @param bis the input stream
 * @return    the Slave-Master Topic Properties that were read
 */
function read(bis) {
    return Codec.readObject(bis, function (input) {
        return KEY_MAP[Codec.readByte(input)];
    }, Codec.readString);
}
exports.read = read;
/**
 * Write Topic Properties to the stream
 *
 * @param bos   the output stream
 * @param value the Topic Properties to be written
 */
function write(bos, value) {
    Codec.writeObject(bos, value, function (output, k) {
        Codec.writeByte(output, KEY_MAP[k]);
    }, Codec.writeString);
}
exports.write = write;
